@@ -11,7 +11,26 @@ knearneigh <- function(x, k=1, longlat=NULL, RANN=TRUE)
            longlat <- TRUE
         } else longlat <- FALSE
         x <- coordinates(x)[, 1:2]
-    } else if (is.null(longlat) || !is.logical(longlat)) longlat <- FALSE
+    } else {
+      if (inherits(x, "sf")) {
+          if (is.null(row.names)) row.names <- row.names(x)
+          x <- sf::st_geometry(x)
+      }
+      if (inherits(x, "sfc")) {
+         if (!is.null(longlat))
+             warning("dnearneigh: longlat argument overrides object")
+         if (!inherits(x, "sfc_POINT"))
+             stop("Point geometries required")
+         if (attr(x, "n_empty") > 0L) 
+             stop("Empty geometries found")
+         if ((is.null(longlat) || !is.logical(longlat)) 
+ 	     && !is.na(sf::st_is_longlat(x)) && sf::st_is_longlat(x)) {
+             longlat <- TRUE
+         } else longlat <- FALSE
+         x <- sf::st_coordinates(x)
+      }
+    } 
+    if (is.null(longlat) || !is.logical(longlat)) longlat <- FALSE
     if (!is.numeric(x)) stop("knearneigh: data non-numeric")
     if (!is.matrix(x)) stop("knearneigh: data not in matrix form")
     stopifnot(ncol(x) == 2)
@@ -26,12 +45,16 @@ knearneigh <- function(x, k=1, longlat=NULL, RANN=TRUE)
     dimension <- ncol(x)
     if (dimension != 2) stop("knearneigh: only 2D data accepted")
     if (k >= np) stop("knearneigh: fewer data points than k")
+    if (k > (np/3)) warning("k greater than one-third of the number of data points")
 # modified 140117 to handle zerodist points
 # (previous fix only worked for pairs and k>1)
     zd <- zerodist(SpatialPoints(x))
     if (!(nrow(zd) == 0)) warning("knearneigh: identical points found")
-    if (RANN && !longlat && (nrow(zd) == 0) && 
-        (requireNamespace("RANN", quietly = TRUE)) ) {
+    useRANN <- RANN
+    if (longlat) useRANN <- FALSE
+    if (nrow(zd) > 0) useRANN <- FALSE
+    if (useRANN && !requireNamespace("RANN", quietly = TRUE)) useRANN <- FALSE
+    if (useRANN) {
 #        xx <- cbind(x, out=rep(0, nrow(x)))
 #        out <- as.matrix(nn(xx, p=k)$nn.idx)
 # nn() retired 130722
